package genutils

import (
	"unicode"

	"github.com/dave/jennifer/jen"
)

// NewFile creates a new file with the appropriate header comments.
func NewFile(pkgname string) *jen.File {
	gen := jen.NewFile(pkgname)
	gen.HeaderComment("Code generated by ./cmd/internal. DO NOT EDIT.")
	return gen
}

// NewFilePath creates a new file with the appropriate header comments.
func NewFilePath(pkgpath string) *jen.File {
	gen := jen.NewFilePath(pkgpath)
	gen.HeaderComment("Code generated by ./cmd/internal. DO NOT EDIT.")
	return gen
}

type Qualer interface {
	Qual() (path, name string)
}

// GenerateType generates a jen.Qual from the given Qualer.
func GenerateType(t Qualer) jen.Code {
	path, name := t.Qual()
	if path == "" {
		return jen.Id(name)
	}
	return jen.Qual(path, name)
}

// GenerateExternType generates a jen.Qual from the given Qualer, except if
// the path is empty, root is used instead.
func GenerateExternType(root string, t Qualer) jen.Code {
	path, name := t.Qual()
	if path == "" {
		return jen.Qual(root, name)
	}
	return jen.Qual(path, name)
}

// RecvName is used to get the receiver variable name. It returns the first
// letter lower-cased. It does NOT do length checking. It only works with ASCII.
func RecvName(name string) string {
	return string(unicode.ToLower(rune(name[0])))
}
